@@ -3,6 +3,8 @@ Tests for bitarray
 
 Author: Ilan Schnell
 """
+from __future__ import absolute_import
+
 import os
 import sys
 import unittest
@@ -30,7 +32,7 @@ else:
     range = xrange
 
 
-from bitarray import (bitarray, frozenbitarray, bitdiff, bits2bytes,
+from bitarray import (bitarray, frozenbitarray, bits2bytes,
                       get_default_endian, _set_default_endian,
                       _sysinfo, __version__)
 
@@ -75,7 +77,8 @@ class Util(object):
         return slicelength
 
     def check_obj(self, a):
-        self.assertEqual(repr(type(a)), "<class 'bitarray.bitarray'>")
+        self.assertEqual(repr(type(a)), "<%s 'bitarray.bitarray'>" %
+                         ('class' if is_py3k else 'type'))
         unused = 8 * a.buffer_info()[1] - len(a)
         self.assertTrue(0 <= unused < 8)
         self.assertEqual(unused, a.buffer_info()[3])
@@ -87,14 +90,19 @@ class Util(object):
         self.check_obj(b)
 
     def assertStopIteration(self, it):
-        if is_py3k:
-            return
-        self.assertRaises(StopIteration, it.next)
+        self.assertRaises(StopIteration, next, it)
 
     if sys.version_info[:2] == (2, 6):
         def assertIsInstance(self, o, t):
             self.assertTrue(isinstance(o, t))
 
+    def assertRaisesMessage(self, excClass, msg, callable, *args, **kwargs):
+        try:
+            callable(*args, **kwargs)
+        except excClass as e:
+            if msg != str(e):
+                raise AssertionError("message: %s\n got: %s" % (msg, e))
+
 # ---------------------------------------------------------------------------
 
 class TestsModuleFunctions(unittest.TestCase, Util):
@@ -134,11 +142,9 @@ class TestsModuleFunctions(unittest.TestCase, Util):
             self.assertIsInstance(endian, str)
 
     def test_bits2bytes(self):
-        for arg in ['foo', [], None, {}]:
+        for arg in 'foo', [], None, {}, 187.0, -4.0:
             self.assertRaises(TypeError, bits2bytes, arg)
 
-        self.assertRaises(TypeError, bits2bytes, 187.0)
-        self.assertRaises(TypeError, bits2bytes, -4.0)
         self.assertRaises(TypeError, bits2bytes)
         self.assertRaises(TypeError, bits2bytes, 1, 2)
 
@@ -198,14 +204,13 @@ class CreateObjectTests(unittest.TestCase, Util):
     def test_endian_wrong(self):
         self.assertRaises(TypeError, bitarray.__new__, bitarray, endian=0)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, endian='')
-        self.assertRaises(ValueError,
-                          bitarray.__new__, bitarray, endian='foo')
-        try:
-            bitarray(Ellipsis)
-        except TypeError as e:
-            self.assertTrue('ellipsis' in str(e))
-        else:
-            raise SystemError("we shouldn'e get here")
+        self.assertRaisesMessage(
+            ValueError,
+            "bit endianness must be either 'little' or 'big', got: 'foo'",
+            bitarray.__new__, bitarray, endian='foo')
+        self.assertRaisesMessage(TypeError,
+                                 "'ellipsis' object is not iterable",
+                                 bitarray.__new__, bitarray, Ellipsis)
 
     def test_integers(self):
         for n in range(50):
@@ -341,25 +346,25 @@ class CreateObjectTests(unittest.TestCase, Util):
         c = bitarray('11100001', endian='big')
         self.assertEqual(a, c)
 
-    def test_None(self):
-        self.assertEQUAL(bitarray(), bitarray(0))
-        self.assertEQUAL(bitarray(), bitarray(None))
+    def test_create_empty(self):
+        for x in None, 0, '', list(), tuple(), set(), dict():
+            a = bitarray(x)
+            self.assertEqual(len(a), 0)
+            self.assertEQUAL(a, bitarray())
 
-    def test_WrongArgs(self):
-        self.assertRaises(TypeError, bitarray.__new__, bitarray, 'A', 42, 69)
-
-        self.assertRaises(TypeError, bitarray.__new__, bitarray, Ellipsis)
-        self.assertRaises(TypeError, bitarray.__new__, bitarray, slice(0))
-
-        self.assertRaises(TypeError, bitarray.__new__, bitarray, 2.345)
-        self.assertRaises(TypeError, bitarray.__new__, bitarray, 4+3j)
-
-        self.assertRaises(TypeError, bitarray.__new__, bitarray, '', 0, 42)
+    def test_wrong_args(self):
+        # wrong types
+        for x in False, True, Ellipsis, slice(0), 0.0, 0 + 0j:
+            self.assertRaises(TypeError, bitarray.__new__, bitarray, x)
+        # wrong values
+        for x in -1, 'A':
+            self.assertRaises(ValueError, bitarray.__new__, bitarray, x)
         # test second (endian) argument
         self.assertRaises(TypeError, bitarray.__new__, bitarray, 0, None)
         self.assertRaises(TypeError, bitarray.__new__, bitarray, 0, 0)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, 0, 'foo')
-
+        # too many args
+        self.assertRaises(TypeError, bitarray.__new__, bitarray, 0, 'big', 0)
 
 tests.append(CreateObjectTests)
 
@@ -417,11 +422,10 @@ class MetaDataTests(unittest.TestCase, Util):
         a = bitarray(endian='big')
         self.assertEqual(a.endian(), 'big')
 
-    def test_length(self):
+    def test_len(self):
         for n in range(100):
             a = bitarray(n)
             self.assertEqual(len(a), n)
-            self.assertEqual(a.length(), n)
 
 
 tests.append(MetaDataTests)
@@ -446,7 +450,7 @@ class SliceTests(unittest.TestCase, Util):
         a = bitarray('1100010')
         for i, b in enumerate([True, True, False, False, False, True, False]):
             self.assertEqual(a[i], b)
-            self.assertEqual(a[i-7], b)
+            self.assertEqual(a[i - 7], b)
         self.assertRaises(IndexError, a.__getitem__,  7)
         self.assertRaises(IndexError, a.__getitem__, -8)
 
@@ -619,6 +623,9 @@ class SliceTests(unittest.TestCase, Util):
         self.assertEqual(a, bitarray('1010110'))
         del a[::3] #                  ^  ^  ^
         self.assertEqual(a, bitarray('0111'))
+        a = bitarray('10101100' '101101111')
+        del a[5:-3:3] #    ^     ^  ^
+        self.assertEqual(a, bitarray('1010100' '0101111'))
         a = bitarray('10101100' '1011011')
         del a[:-9:-2] #          ^ ^ ^ ^
         self.assertEqual(a, bitarray('10101100' '011'))
@@ -629,14 +636,12 @@ class SliceTests(unittest.TestCase, Util):
             for dum in range(10):
                 step = self.rndsliceidx(la) or None
                 s = slice(self.rndsliceidx(la), self.rndsliceidx(la), step)
-                c = bitarray(a)
-                d = c
+                c = a.copy()
                 del c[s]
-                self.assertTrue(c is d)
                 self.check_obj(c)
-                cc = a.tolist()
-                del cc[s]
-                self.assertEQUAL(c, bitarray(cc, endian=c.endian()))
+                c_lst = a.tolist()
+                del c_lst[s]
+                self.assertEQUAL(c, bitarray(c_lst, endian=c.endian()))
 
 
 tests.append(SliceTests)
@@ -691,7 +696,23 @@ class MiscTests(unittest.TestCase, Util):
         b = bitarray('01010111000')
         self.assertEqual(a, b)
 
-    def test_compare(self):
+    def test_compare_eq_ne(self):
+        self.assertTrue(bitarray(0, 'big') == bitarray(0, 'little'))
+        self.assertFalse(bitarray(0, 'big') != bitarray(0, 'little'))
+
+        for n in range(1, 20):
+            a = bitarray(n, 'little')
+            a.setall(1)
+            for endian in 'little', 'big':
+                b = bitarray(n, endian)
+                b.setall(1)
+                self.assertTrue(a == b)
+                self.assertFalse(a != b)
+                b[n - 1] = not b[n - 1]  # flip last bit
+                self.assertTrue(a != b)
+                self.assertFalse(a == b)
+
+    def test_compare_random(self):
         for a in self.randombitarrays():
             aa = a.tolist()
             for b in self.randombitarrays():
@@ -776,23 +797,23 @@ class MiscTests(unittest.TestCase, Util):
         self.assertEqual(a.tobytes(), b.tobytes())
 
     def test_pickle(self):
-        for v in range(3):
-            for a in self.randombitarrays():
-                b = pickle.loads(pickle.dumps(a, v))
-                self.assertFalse(b is a)
-                self.assertEQUAL(a, b)
+        for a in self.randombitarrays():
+            b = pickle.loads(pickle.dumps(a))
+            self.assertFalse(b is a)
+            self.assertEQUAL(a, b)
 
     def test_overflow(self):
         if _sysinfo()[0] == 8:
             return
 
-        self.assertRaises(OverflowError, bitarray.__new__,
-                          bitarray, 2**34 + 1)
+        a = bitarray(2**31 - 1);
+        self.assertRaises(OverflowError, bitarray.append, a, True)
+        self.assertRaises(IndexError, bitarray.__new__, bitarray, 2**31)
 
         a = bitarray(10 ** 6)
         self.assertRaises(OverflowError, a.__imul__, 17180)
 
-    def test_unicode1(self):
+    def test_unicode_create(self):
         a = bitarray(unicode())
         self.assertEqual(a, bitarray())
 
@@ -803,7 +824,7 @@ class MiscTests(unittest.TestCase, Util):
             b = bitarray(unicode(a.to01()))
             self.assertEqual(a, b)
 
-    def test_unicode2(self):
+    def test_unicode_extend(self):
         a = bitarray()
         a.extend(unicode())
         self.assertEqual(a, bitarray())
@@ -928,9 +949,9 @@ tests.append(SpecialMethodTests)
 
 # ---------------------------------------------------------------------------
 
-class NumberTests(unittest.TestCase, Util):
+class SequenceMethodsTests(unittest.TestCase, Util):
 
-    def test_add(self):
+    def test_concat(self):
         c = bitarray('001') + bitarray('110')
         self.assertEQUAL(c, bitarray('001110'))
 
@@ -949,11 +970,12 @@ class NumberTests(unittest.TestCase, Util):
         a = bitarray()
         self.assertRaises(TypeError, a.__add__, 42)
 
-
-    def test_iadd(self):
+    def test_inplace_concat(self):
         c = bitarray('001')
         c += bitarray('110')
-        self.assertEQUAL(c, bitarray('001110'))
+        self.assertEqual(c, bitarray('001110'))
+        c += '111'
+        self.assertEqual(c, bitarray('001110111'))
 
         for a in self.randombitarrays():
             for b in self.randombitarrays():
@@ -969,8 +991,7 @@ class NumberTests(unittest.TestCase, Util):
         a = bitarray()
         self.assertRaises(TypeError, a.__iadd__, 42)
 
-
-    def test_mul(self):
+    def test_repeat(self):
         for c in [0 * bitarray(),
                   0 * bitarray('1001111'),
                   -1 * bitarray('100110'),
@@ -997,8 +1018,7 @@ class NumberTests(unittest.TestCase, Util):
         a = bitarray()
         self.assertRaises(TypeError, a.__mul__, None)
 
-
-    def test_imul(self):
+    def test_inplace_repeat(self):
         c = bitarray('1101110011')
         idc = id(c)
         c *= 0
@@ -1010,7 +1030,7 @@ class NumberTests(unittest.TestCase, Util):
         self.assertEQUAL(c, bitarray('110110110'))
 
         for a in self.randombitarrays():
-            for n in range(-10, 10):
+            for n in range(-3, 5):
                 b = a.copy()
                 idb = id(b)
                 b *= n
@@ -1021,12 +1041,66 @@ class NumberTests(unittest.TestCase, Util):
         a = bitarray()
         self.assertRaises(TypeError, a.__imul__, None)
 
+    def test_contains_simple(self):
+        a = bitarray()
+        self.assertFalse(False in a)
+        self.assertFalse(True in a)
+        self.assertTrue(bitarray() in a)
+        a.append(True)
+        self.assertTrue(True in a)
+        self.assertFalse(False in a)
+        a = bitarray([False])
+        self.assertTrue(False in a)
+        self.assertFalse(True in a)
+        a.append(True)
+        self.assertTrue(0 in a)
+        self.assertTrue(1 in a)
+        if not is_py3k:
+            self.assertTrue(long(0) in a)
+            self.assertTrue(long(1) in a)
 
-tests.append(NumberTests)
+    def test_contains_errors(self):
+        a = bitarray()
+        self.assertEqual(a.__contains__(1), False)
+        a.append(1)
+        self.assertEqual(a.__contains__(1), True)
+        a = bitarray('0011')
+        self.assertEqual(a.__contains__(bitarray('01')), True)
+        self.assertEqual(a.__contains__(bitarray('10')), False)
+        self.assertRaises(TypeError, a.__contains__, 'asdf')
+        self.assertRaises(ValueError, a.__contains__, 2)
+        self.assertRaises(ValueError, a.__contains__, -1)
+        if not is_py3k:
+            self.assertRaises(ValueError, a.__contains__, long(2))
+
+    def test_contains_range(self):
+        for n in range(2, 50):
+            a = bitarray(n)
+            a.setall(0)
+            self.assertTrue(False in a)
+            self.assertFalse(True in a)
+            a[randint(0, n - 1)] = 1
+            self.assertTrue(True in a)
+            self.assertTrue(False in a)
+            a.setall(1)
+            self.assertTrue(True in a)
+            self.assertFalse(False in a)
+            a[randint(0, n - 1)] = 0
+            self.assertTrue(True in a)
+            self.assertTrue(False in a)
+
+    def test_contains_explicit(self):
+        a = bitarray('011010000001')
+        for s, r in [('', True), ('1', True), ('11', True), ('111', False),
+                     ('011', True), ('0001', True), ('00011', False)]:
+            self.assertEqual(bitarray(s) in a, r)
+
+
+tests.append(SequenceMethodsTests)
 
 # ---------------------------------------------------------------------------
 
-class BitwiseTests(unittest.TestCase, Util):
+class NumberMethodsTests(unittest.TestCase, Util):
 
     def test_misc(self):
         for a in self.randombitarrays():
@@ -1039,6 +1113,18 @@ class BitwiseTests(unittest.TestCase, Util):
             b &= d
             self.assertEqual(~b, a)
 
+    def test_size_and_endianness(self):
+        a = bitarray('11001')
+        b = bitarray('100111')
+        self.assertRaises(ValueError, a.__and__, b)
+        for x in a.__or__, a.__xor__, a.__iand__, a.__ior__, a.__ixor__:
+            self.assertRaises(ValueError, x, b)
+        a = bitarray('11001', 'big')
+        b = bitarray('10011', 'little')
+        self.assertRaises(ValueError, a.__and__, b)
+        for x in a.__or__, a.__xor__, a.__iand__, a.__ior__, a.__ixor__:
+            self.assertRaises(ValueError, x, b)
+
     def test_and(self):
         a = bitarray('11001')
         b = bitarray('10011')
@@ -1046,8 +1132,8 @@ class BitwiseTests(unittest.TestCase, Util):
 
         b = bitarray('1001')
         self.assertRaises(ValueError, a.__and__, b)  # not same length
-
-        self.assertRaises(TypeError, a.__and__, 42)
+        if is_py3k:  # XXX: note sure why this is failing on Py27
+            self.assertRaises(TypeError, a.__and__, 42)
 
     def test_iand(self):
         a =  bitarray('110010110')
@@ -1109,68 +1195,7 @@ class BitwiseTests(unittest.TestCase, Util):
             self.check_obj(c)
 
 
-tests.append(BitwiseTests)
-
-# ---------------------------------------------------------------------------
-
-class SequenceTests(unittest.TestCase, Util):
-
-    def test_contains1(self):
-        a = bitarray()
-        self.assertFalse(False in a)
-        self.assertFalse(True in a)
-        self.assertTrue(bitarray() in a)
-        a.append(True)
-        self.assertTrue(True in a)
-        self.assertFalse(False in a)
-        a = bitarray([False])
-        self.assertTrue(False in a)
-        self.assertFalse(True in a)
-        a.append(True)
-        self.assertTrue(0 in a)
-        self.assertTrue(1 in a)
-        if not is_py3k:
-            self.assertTrue(long(0) in a)
-            self.assertTrue(long(1) in a)
-
-    def test_contains2(self):
-        a = bitarray()
-        self.assertEqual(a.__contains__(1), False)
-        a.append(1)
-        self.assertEqual(a.__contains__(1), True)
-        a = bitarray('0011')
-        self.assertEqual(a.__contains__(bitarray('01')), True)
-        self.assertEqual(a.__contains__(bitarray('10')), False)
-        self.assertRaises(TypeError, a.__contains__, 'asdf')
-        self.assertRaises(ValueError, a.__contains__, 2)
-        self.assertRaises(ValueError, a.__contains__, -1)
-        if not is_py3k:
-            self.assertRaises(ValueError, a.__contains__, long(2))
-
-    def test_contains3(self):
-        for n in range(2, 50):
-            a = bitarray(n)
-            a.setall(0)
-            self.assertTrue(False in a)
-            self.assertFalse(True in a)
-            a[randint(0, n - 1)] = 1
-            self.assertTrue(True in a)
-            self.assertTrue(False in a)
-            a.setall(1)
-            self.assertTrue(True in a)
-            self.assertFalse(False in a)
-            a[randint(0, n - 1)] = 0
-            self.assertTrue(True in a)
-            self.assertTrue(False in a)
-
-    def test_contains4(self):
-        a = bitarray('011010000001')
-        for s, r in [('', True), ('1', True), ('11', True), ('111', False),
-                     ('011', True), ('0001', True), ('00011', False)]:
-            self.assertEqual(bitarray(s) in a, r)
-
-
-tests.append(SequenceTests)
+tests.append(NumberMethodsTests)
 
 # ---------------------------------------------------------------------------
 
@@ -1592,20 +1617,19 @@ class MethodTests(unittest.TestCase, Util):
         it = a.itersearch(bitarray('1'))
         self.assertIsInstance(type(it), type)
 
-    def test_fill(self):
-        a = bitarray('')
-        self.assertEqual(a.fill(), 0)
-        self.assertEqual(len(a), 0)
+    def test_fill_simple(self):
+        for endian in 'little', 'big':
+            a = bitarray(endian=endian)
+            self.assertEqual(a.fill(), 0)
+            self.assertEqual(len(a), 0)
 
-        a = bitarray('101')
-        b = a
-        self.assertEqual(a.fill(), 5)
-        self.assertEQUAL(a, bitarray('10100000'))
-        self.assertEqual(a.fill(), 0)
-        self.assertEQUAL(a, bitarray('10100000'))
-        self.assertTrue(a is b)
+            a = bitarray('101', endian)
+            self.assertEqual(a.fill(), 5)
+            self.assertEqual(a, bitarray('10100000'))
+            self.assertEqual(a.fill(), 0)
+            self.assertEqual(a, bitarray('10100000'))
 
-    def test_random(self):
+    def test_fill_random(self):
         for a in self.randombitarrays():
             b = a.copy()
             res = b.fill()
@@ -1878,9 +1902,9 @@ class BytesTests(unittest.TestCase, Util):
             self.assertEqual(a.tobytes(), s)
 
 
-    def test_unpack(self):
+    def test_unpack_simple(self):
         a = bitarray('01')
-        self.assertIsInstance(a.unpack(), bytes if is_py3k else str)
+        self.assertIsInstance(a.unpack(), bytes)
         self.assertEqual(a.unpack(), b'\x00\xff')
         self.assertEqual(a.unpack(b'A'), b'A\xff')
         self.assertEqual(a.unpack(b'0', b'1'), b'01')
@@ -1888,36 +1912,51 @@ class BytesTests(unittest.TestCase, Util):
         self.assertEqual(a.unpack(zero=b'A'), b'A\xff')
         self.assertEqual(a.unpack(one=b't', zero=b'f'), b'ft')
 
-        self.assertRaises(TypeError, a.unpack, b'a', zero=b'b')
-        self.assertRaises(TypeError, a.unpack, foo=b'b')
-
+    def test_unpack_random(self):
         for a in self.randombitarrays():
-            self.assertEqual(a.unpack(b'0', b'1'), a.to01().encode())
-
+            self.assertEqual(a.unpack(b'0', b'1'),
+                             a.to01().encode())
+            # round trip
             b = bitarray()
             b.pack(a.unpack())
             self.assertEqual(b, a)
-
+            # round trip with invert
             b = bitarray()
             b.pack(a.unpack(b'\x01', b'\x00'))
             b.invert()
             self.assertEqual(b, a)
 
+    def test_unpack_errors(self):
+        a = bitarray('01')
+        self.assertRaises(TypeError, a.unpack, b'')
+        self.assertRaises(TypeError, a.unpack, b'0', b'')
+        self.assertRaises(TypeError, a.unpack, b'a', zero=b'b')
+        self.assertRaises(TypeError, a.unpack, foo=b'b')
+        self.assertRaises(TypeError, a.unpack, one=b'aa', zero=b'b')
+        if is_py3k:
+            self.assertRaises(TypeError, a.unpack, '0')
+            self.assertRaises(TypeError, a.unpack, one='a')
+            self.assertRaises(TypeError, a.unpack, b'0', '1')
 
-    def test_pack(self):
-        a = bitarray()
-        a.pack(b'\x00')
-        self.assertEqual(a, bitarray('0'))
-        a.pack(b'\xff')
-        self.assertEqual(a, bitarray('01'))
-        a.pack(b'\x01\x00\x7a')
-        self.assertEqual(a, bitarray('01101'))
+    def test_pack_simple(self):
+        for endian in 'little', 'big':
+            _set_default_endian(endian)
+            a = bitarray()
+            a.pack(b'\x00')
+            self.assertEqual(a, bitarray('0'))
+            a.pack(b'\xff')
+            self.assertEqual(a, bitarray('01'))
+            a.pack(b'\x01\x00\x7a')
+            self.assertEqual(a, bitarray('01101'))
 
+    def test_pack_random(self):
         a = bitarray()
         for n in range(256):
             a.pack(bytes(bytearray([n])))
         self.assertEqual(a, bitarray('0' + 255 * '1'))
 
+    def test_pack_errors(self):
+        a = bitarray()
         self.assertRaises(TypeError, a.pack, 0)
         if is_py3k:
             self.assertRaises(TypeError, a.pack, '1')
@@ -1938,18 +1977,22 @@ class FileTests(unittest.TestCase, Util):
     def tearDown(self):
         shutil.rmtree(self.tmpdir)
 
+    def read_file(self):
+        with open(self.tmpfname, 'rb') as fi:
+            return fi.read()
+
     def assertFileSize(self, size):
         self.assertEqual(os.path.getsize(self.tmpfname), size)
 
 
     def test_pickle(self):
-        for v in range(3):
-            for a in self.randombitarrays():
-                with open(self.tmpfname, 'wb') as fo:
-                    pickle.dump(a, fo, v)
-                b = pickle.load(open(self.tmpfname, 'rb'))
-                self.assertFalse(b is a)
-                self.assertEQUAL(a, b)
+        for a in self.randombitarrays():
+            with open(self.tmpfname, 'wb') as fo:
+                pickle.dump(a, fo)
+            with open(self.tmpfname, 'rb') as fi:
+                b = pickle.load(fi)
+            self.assertFalse(b is a)
+            self.assertEQUAL(a, b)
 
     def test_shelve(self):
         if not shelve or hasattr(sys, 'gettotalrefcount'):
@@ -1977,7 +2020,8 @@ class FileTests(unittest.TestCase, Util):
         self.assertFileSize(0)
 
         a = bitarray()
-        a.fromfile(open(self.tmpfname, 'rb'))
+        with open(self.tmpfname, 'rb') as fi:
+            a.fromfile(fi)
         self.assertEqual(a, bitarray())
 
     def test_fromfile_Foo(self):
@@ -1986,11 +2030,13 @@ class FileTests(unittest.TestCase, Util):
         self.assertFileSize(3)
 
         a = bitarray(endian='big')
-        a.fromfile(open(self.tmpfname, 'rb'))
+        with open(self.tmpfname, 'rb') as fi:
+            a.fromfile(fi)
         self.assertEqual(a, bitarray('01000110' '01101111' '01101111'))
 
         a = bitarray(endian='little')
-        a.fromfile(open(self.tmpfname, 'rb'))
+        with open(self.tmpfname, 'rb') as fi:
+            a.fromfile(fi)
         self.assertEqual(a, bitarray('01100010' '11110110' '11110110'))
 
     def test_fromfile_wrong_args(self):
@@ -2010,11 +2056,11 @@ class FileTests(unittest.TestCase, Util):
             fo.write(b'0123456789')
         self.assertFileSize(10)
 
-        if is_py3k:
-            a = bitarray()
-            with open(self.tmpfname, 'wb') as fi:
-                self.assertRaises(Exception, a.fromfile, fi)
+        a = bitarray()
+        with open(self.tmpfname, 'wb') as fi:
+            self.assertRaises(Exception, a.fromfile, fi)
 
+        if is_py3k:
             with open(self.tmpfname, 'r') as fi:
                 self.assertRaises(TypeError, a.fromfile, fi)
 
@@ -2025,24 +2071,28 @@ class FileTests(unittest.TestCase, Util):
                 fo.write(data)
 
             a = bitarray()
-            a.fromfile(open(self.tmpfname, 'rb'))
+            with open(self.tmpfname, 'rb') as fi:
+                a.fromfile(fi)
             self.assertEqual(len(a), 8 * N)
             self.assertEqual(a.buffer_info()[1], N)
             self.assertEqual(a.tobytes(), data)
 
-    def test_fromfile_extend_exsiting(self):
+    def test_fromfile_extend_existing(self):
         with open(self.tmpfname, 'wb') as fo:
             fo.write(b'Foo')
 
         foo_le = '011000101111011011110110'
         a = bitarray('1', endian='little')
-        a.fromfile(open(self.tmpfname, 'rb'))
+        with open(self.tmpfname, 'rb') as fi:
+            a.fromfile(fi)
+
         self.assertEqual(a, bitarray('1' + foo_le))
 
         for n in range(20):
             a = bitarray(n, endian='little')
             a.setall(1)
-            a.fromfile(open(self.tmpfname, 'rb'))
+            with open(self.tmpfname, 'rb') as fi:
+                a.fromfile(fi)
             self.assertEqual(a, bitarray(n * '1' + foo_le))
 
     def test_fromfile_n(self):
@@ -2104,8 +2154,7 @@ class FileTests(unittest.TestCase, Util):
         self.assertEQUAL(a, b)
 
         self.assertFileSize(3)
-        with open(self.tmpfname, 'rb') as fi:
-            self.assertEqual(fi.read(), b'Foo')
+        self.assertEqual(self.read_file(), b'Foo')
 
     def test_tofile_random(self):
         for a in self.randombitarrays():
@@ -2113,7 +2162,7 @@ class FileTests(unittest.TestCase, Util):
                 a.tofile(fo)
             n = bits2bytes(len(a))
             self.assertFileSize(n)
-            raw = open(self.tmpfname, 'rb').read()
+            raw = self.read_file()
             self.assertEqual(len(raw), n)
             self.assertEqual(raw, a.tobytes())
 
@@ -2144,7 +2193,7 @@ class FileTests(unittest.TestCase, Util):
             a.tofile(f)
         self.assertFileSize(n)
 
-        raw = open(self.tmpfname, 'rb').read()
+        raw = self.read_file()
         self.assertEqual(len(raw), n)
         self.assertEqual(raw, a.tobytes())
 
@@ -2154,7 +2203,7 @@ class FileTests(unittest.TestCase, Util):
             with open(self.tmpfname, 'wb') as fo:
                 a.tofile(fo)
 
-            raw = open(self.tmpfname, 'rb').read()
+            raw = self.read_file()
             self.assertEqual(len(raw), bits2bytes(len(a)))
             # when we the the unused bits in a, we can compare
             a.fill()
@@ -2319,18 +2368,16 @@ class PrefixCodeTests(unittest.TestCase, Util):
         d = {'a': bitarray('0'), 'b': bitarray('111')}
         a = bitarray('011')
         it = a.iterdecode(d)
-        if not is_py3k:
-            self.assertEqual(it.next(), 'a')
-            self.assertRaises(ValueError, it.next)
+        self.assertEqual(next(it), 'a')
+        self.assertRaisesMessage(ValueError, "decoding not terminated",
+                                 next, it)
         self.assertEqual(a, bitarray('011'))
-        self.assertEqual(d, {'a': bitarray('0'), 'b': bitarray('111')})
 
     def test_iterdecode_buggybitarray(self):
         d = {'a': bitarray('0')}
         a = bitarray('1')
         it = a.iterdecode(d)
-        if not is_py3k:
-            self.assertRaises(ValueError, it.next)
+        self.assertRaises(ValueError, next, it)
         self.assertEqual(a, bitarray('1'))
         self.assertEqual(d, {'a': bitarray('0')})
 
@@ -2344,8 +2391,7 @@ class PrefixCodeTests(unittest.TestCase, Util):
         d = {'a': bitarray('00'), 'b': bitarray('01')}
         a = bitarray('1')
         it = a.iterdecode(d)
-        if not is_py3k:
-            self.assertRaises(ValueError, it.next)
+        self.assertRaises(ValueError, next, it)
         self.assertEqual(a, bitarray('1'))
 
     def test_decode_ambiguous_code(self):
@@ -2356,8 +2402,10 @@ class PrefixCodeTests(unittest.TestCase, Util):
             {'a': bitarray('0'), 'b': bitarray('11'), 'c': bitarray('111')},
         ]:
             a = bitarray()
-            self.assertRaises(ValueError, a.decode, d)
-            self.assertRaises(ValueError, a.iterdecode, d)
+            self.assertRaisesMessage(ValueError, "prefix code ambiguous",
+                                     a.decode, d)
+            self.assertRaisesMessage(ValueError, "prefix code ambiguous",
+                                     a.iterdecode, d)
 
     def test_miscitems(self):
         d = {None : bitarray('00'),
@@ -2462,6 +2510,18 @@ class BufferInterfaceTests(unittest.TestCase):
         v[301:304] = b'ABC'
         self.assertEqual(a[300 * 8 : 305 * 8].tobytes(), b'\x00ABC\x00')
 
+    def test_write_py3(self):
+        if not is_py3k:
+            return
+        a = bitarray(40)
+        a.setall(0)
+        m = memoryview(a)
+        v = m[1:4]
+        v[0] = 65
+        v[1] = 66
+        v[2] = 67
+        self.assertEqual(a.tobytes(), b'\x00ABC\x00')
+
 if sys.version_info[:2] >= (2, 7):
     tests.append(BufferInterfaceTests)
 
@@ -2488,11 +2548,10 @@ class TestsFrozenbitarray(unittest.TestCase, Util):
         self.assertEqual(b, a)
         self.assertEqual(repr(type(b)), "<class 'bitarray.frozenbitarray'>")
         self.assertEqual(len(b), 7)
-        self.assertEqual(b.length(), 7)
         self.assertEqual(b.all(), False)
         self.assertEqual(b.any(), True)
 
-    def test_init_bitarray(self):
+    def test_init_from_bitarray(self):
         for a in self.randombitarrays():
             b = frozenbitarray(a)
             self.assertFalse(b is a)
@@ -2541,6 +2600,15 @@ class TestsFrozenbitarray(unittest.TestCase, Util):
         a.extend(b)
         self.assertEqual(a, bitarray('1100011'))
 
+    def test_pickle(self):
+        for a in self.randombitarrays():
+            f = frozenbitarray(a)
+            g = pickle.loads(pickle.dumps(f))
+            self.assertEqual(f, g)
+            self.assertEqual(f.endian(), g.endian())
+            self.assertTrue(str(g).startswith('frozenbitarray'))
+
+
 tests.append(TestsFrozenbitarray)
 
 # ---------------------------------------------------------------------------
